@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2020 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2020 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qcborstreamreader.h"
 
@@ -44,9 +8,11 @@
 
 #include <private/qbytearray_p.h>
 #include <private/qnumeric_p.h>
-#include <private/qutfcodec_p.h>
+#include <private/qstringconverter_p.h>
+#include <qiodevice.h>
 #include <qdebug.h>
 #include <qstack.h>
+#include <qvarlengtharray.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -70,32 +36,31 @@ QT_WARNING_POP
 
 static CborError _cbor_value_dup_string(const CborValue *, void **, size_t *, CborValue *)
 {
-    Q_UNREACHABLE();
-    return CborErrorInternalError;
+    Q_UNREACHABLE_RETURN(CborErrorInternalError);
 }
-static CborError Q_DECL_UNUSED cbor_value_get_half_float_as_float(const CborValue *, float *)
+[[maybe_unused]] static CborError cbor_value_get_half_float_as_float(const CborValue *, float *)
 {
-    Q_UNREACHABLE();
-    return CborErrorInternalError;
+    Q_UNREACHABLE_RETURN(CborErrorInternalError);
 }
 
 // confirm our constants match TinyCBOR's
-Q_STATIC_ASSERT(int(QCborStreamReader::UnsignedInteger) == CborIntegerType);
-Q_STATIC_ASSERT(int(QCborStreamReader::ByteString) == CborByteStringType);
-Q_STATIC_ASSERT(int(QCborStreamReader::TextString) == CborTextStringType);
-Q_STATIC_ASSERT(int(QCborStreamReader::Array) == CborArrayType);
-Q_STATIC_ASSERT(int(QCborStreamReader::Map) == CborMapType);
-Q_STATIC_ASSERT(int(QCborStreamReader::Tag) == CborTagType);
-Q_STATIC_ASSERT(int(QCborStreamReader::SimpleType) == CborSimpleType);
-Q_STATIC_ASSERT(int(QCborStreamReader::HalfFloat) == CborHalfFloatType);
-Q_STATIC_ASSERT(int(QCborStreamReader::Float) == CborFloatType);
-Q_STATIC_ASSERT(int(QCborStreamReader::Double) == CborDoubleType);
-Q_STATIC_ASSERT(int(QCborStreamReader::Invalid) == CborInvalidType);
+static_assert(int(QCborStreamReader::UnsignedInteger) == CborIntegerType);
+static_assert(int(QCborStreamReader::ByteString) == CborByteStringType);
+static_assert(int(QCborStreamReader::TextString) == CborTextStringType);
+static_assert(int(QCborStreamReader::Array) == CborArrayType);
+static_assert(int(QCborStreamReader::Map) == CborMapType);
+static_assert(int(QCborStreamReader::Tag) == CborTagType);
+static_assert(int(QCborStreamReader::SimpleType) == CborSimpleType);
+static_assert(int(QCborStreamReader::HalfFloat) == CborHalfFloatType);
+static_assert(int(QCborStreamReader::Float) == CborFloatType);
+static_assert(int(QCborStreamReader::Double) == CborDoubleType);
+static_assert(int(QCborStreamReader::Invalid) == CborInvalidType);
 
 /*!
    \class QCborStreamReader
    \inmodule QtCore
    \ingroup cbor
+   \ingroup qtserialization
    \reentrant
    \since 5.12
 
@@ -107,7 +72,7 @@ Q_STATIC_ASSERT(int(QCborStreamReader::Invalid) == CborInvalidType);
    Representation, a very compact form of binary data encoding that is
    compatible with JSON. It was created by the IETF Constrained RESTful
    Environments (CoRE) WG, which has used it in many new RFCs. It is meant to
-   be used alongside the \l{https://tools.ietf.org/html/rfc7252}{CoAP
+   be used alongside the \l{RFC 7252}{CoAP
    protocol}.
 
    QCborStreamReader provides a StAX-like API, similar to that of
@@ -185,7 +150,9 @@ Q_STATIC_ASSERT(int(QCborStreamReader::Invalid) == CborInvalidType);
    parsing from a QByteArray, or reparse(), if it is instead reading directly
    a the QIDOevice that now has more data available (see setDevice()).
 
-   \sa QCborStreamWriter, QCborValue, QXmlStreamReader
+   \sa QCborStreamWriter, QCborValue, QXmlStreamReader,
+       {Parsing and displaying CBOR data}, {Serialization Converter},
+       {Saving and Loading a Game}
  */
 
 /*!
@@ -563,7 +530,7 @@ public:
     CborValue currentElement;
     QCborError lastError = {};
 
-    QByteArray::size_type bufferStart;
+    QByteArray::size_type bufferStart = 0;
     bool corrupt = false;
 
     QCborStreamReaderPrivate(const QByteArray &data)
@@ -645,29 +612,28 @@ public:
         lastError = QCborError { QCborError::Code(int(err)) };
     }
 
-    void updateBufferAfterString(qsizetype offset, qsizetype size)
-    {
-        Q_ASSERT(device);
+    struct ReadStringChunk {
+        union {
+            char *ptr;
+            QByteArray *array;
+            QString *string;
+        };
+        enum { ByteArray = -1, String = -3 };
+        qsizetype maxlen_or_type;
 
-        bufferStart += offset;
-        qsizetype newStart = bufferStart + size;
-        qsizetype remainingInBuffer = buffer.size() - newStart;
-
-        if (remainingInBuffer <= 0) {
-            // We've read from the QIODevice more than what was in the buffer.
-            buffer.truncate(0);
-        } else {
-            // There's still data buffered, but we need to move it around.
-            char *ptr = buffer.data();
-            memmove(ptr, ptr + newStart, remainingInBuffer);
-            buffer.truncate(remainingInBuffer);
-        }
-
-        bufferStart = 0;
-    }
+        ReadStringChunk(char *ptr, qsizetype maxlen) : ptr(ptr), maxlen_or_type(maxlen) {}
+        ReadStringChunk(QByteArray *array) : array(array), maxlen_or_type(ByteArray) {}
+        ReadStringChunk(QString *str) : string(str), maxlen_or_type(String) {}
+        bool isString() const { return maxlen_or_type == String; }
+        bool isByteArray() const { return maxlen_or_type == ByteArray; }
+        bool isPlainPointer() const { return maxlen_or_type >= 0; }
+    };
 
+    static QCborStreamReader::StringResultCode appendStringChunk(QCborStreamReader &reader, QByteArray *data);
+    QCborStreamReader::StringResult<qsizetype> readStringChunk(ReadStringChunk params);
+    qsizetype readStringChunk_byte(ReadStringChunk params, qsizetype len);
+    qsizetype readStringChunk_unicode(ReadStringChunk params, qsizetype utf8len);
     bool ensureStringIteration();
-    QCborStreamReader::StringResult<qsizetype> readStringChunk(char *ptr, qsizetype maxlen);
 };
 
 void qt_cbor_stream_set_error(QCborStreamReaderPrivate *d, QCborError error)
@@ -709,14 +675,14 @@ static CborError qt_cbor_decoder_transfer_string(void *token, const void **userp
 {
     auto self = static_cast<QCborStreamReaderPrivate *>(token);
     Q_ASSERT(offset <= size_t(self->buffer.size()));
-    Q_STATIC_ASSERT(sizeof(size_t) >= sizeof(QByteArray::size_type));
-    Q_STATIC_ASSERT(sizeof(size_t) == sizeof(qsizetype));
+    static_assert(sizeof(size_t) >= sizeof(QByteArray::size_type));
+    static_assert(sizeof(size_t) == sizeof(qsizetype));
 
     // check that we will have enough data from the QIODevice before we advance
     // (otherwise, we'd lose the length information)
     qsizetype total;
     if (len > size_t(std::numeric_limits<QByteArray::size_type>::max())
-            || add_overflow<qsizetype>(offset, len, &total))
+            || qAddOverflow<qsizetype>(offset, len, &total))
         return CborErrorDataTooLarge;
 
     // our string transfer is just saving the offset to the userptr
@@ -786,7 +752,7 @@ inline void QCborStreamReader::preparse()
    \sa addData(), isValid()
  */
 QCborStreamReader::QCborStreamReader()
-    : QCborStreamReader(QByteArray())
+    : d(new QCborStreamReaderPrivate({})), type_(Invalid)
 {
 }
 
@@ -1004,7 +970,7 @@ QCborStreamReader::Type QCborStreamReader::parentContainerType() const
 {
     if (d->containerStack.isEmpty())
         return Invalid;
-    return Type(cbor_value_get_type(&qAsConst(d->containerStack).top()));
+    return Type(cbor_value_get_type(&std::as_const(d->containerStack).top()));
 }
 
 /*!
@@ -1054,19 +1020,18 @@ bool QCborStreamReader::next(int maxRecursion)
             next(maxRecursion - 1);
         if (lastError() == QCborError::NoError)
             leaveContainer();
-    } else if (isString() || isByteArray()) {
-        auto r = _readByteArray_helper();
-        while (r.status == Ok) {
-            if (isString() && r.data.size() > MaxStringSize) {
-                d->handleError(CborErrorDataTooLarge);
-                break;
-            }
-            if (isString() && !QUtf8::isValidUtf8(r.data, r.data.size()).isValidUtf8) {
-                d->handleError(CborErrorInvalidUtf8TextString);
-                break;
-            }
-            r = _readByteArray_helper();
-        }
+    } else if (isByteArray()) {
+        char c;
+        StringResult<qsizetype> r;
+        do {
+            r = readStringChunk(&c, 1);
+        } while (r.status == Ok);
+    } else if (isString()) {
+        // we need to use actual readString so we get UTF-8 validation
+        StringResult<QString> r;
+        do {
+            r = readString();
+        } while (r.status == Ok);
     } else {
         // fixed types
         CborError err = cbor_value_advance_fixed(&d->currentElement);
@@ -1338,34 +1303,22 @@ bool QCborStreamReader::leaveContainer()
  */
 QCborStreamReader::StringResult<QString> QCborStreamReader::_readString_helper()
 {
-    auto r = _readByteArray_helper();
     QCborStreamReader::StringResult<QString> result;
+    auto r = d->readStringChunk(&result.data);
     result.status = r.status;
-
-    if (r.status == Ok) {
-        // See QUtf8::convertToUnicode() a detailed explanation of why this
-        // conversion uses the same number of words or less.
-        CborError err = CborNoError;
-        if (r.data.size() > MaxStringSize) {
-            err = CborErrorDataTooLarge;
-        } else {
-            QTextCodec::ConverterState cs;
-            result.data = QUtf8::convertToUnicode(r.data, r.data.size(), &cs);
-            if (cs.invalidChars != 0 || cs.remainingChars != 0)
-                err = CborErrorInvalidUtf8TextString;
-        }
-
-        if (err) {
-            d->handleError(err);
-            result.data.clear();
-            result.status = Error;
-        }
+    if (r.status == Error) {
+        result.data.clear();
+    } else {
+        Q_ASSERT(r.data == result.data.size());
+        if (r.status == EndOfString && lastError() == QCborError::NoError)
+            preparse();
     }
+
     return result;
 }
 
 /*!
-   \fn QCborStreamReader::StringResult<QString> QCborStreamReader::readByteArray()
+   \fn QCborStreamReader::StringResult<QByteArray> QCborStreamReader::readByteArray()
 
    Decodes one byte array chunk from the CBOR string and returns it. This
    function is used for both regular and chunked contents, so the caller must
@@ -1383,19 +1336,16 @@ QCborStreamReader::StringResult<QString> QCborStreamReader::_readString_helper()
 QCborStreamReader::StringResult<QByteArray> QCborStreamReader::_readByteArray_helper()
 {
     QCborStreamReader::StringResult<QByteArray> result;
-    result.status = Error;
-    qsizetype len = _currentStringChunkSize();
-    if (len < 0)
-        return result;
-    if (len > MaxByteArraySize) {
-        d->handleError(CborErrorDataTooLarge);
-        return result;
+    auto r = d->readStringChunk(&result.data);
+    result.status = r.status;
+    if (r.status == Error) {
+        result.data.clear();
+    } else {
+        Q_ASSERT(r.data == result.data.size());
+        if (r.status == EndOfString && lastError() == QCborError::NoError)
+            preparse();
     }
 
-    result.data.resize(len);
-    auto r = readStringChunk(result.data.data(), len);
-    Q_ASSERT(r.status != Ok || r.data == len);
-    result.status = r.status;
     return result;
 }
 
@@ -1461,14 +1411,29 @@ qsizetype QCborStreamReader::_currentStringChunkSize() const
 QCborStreamReader::StringResult<qsizetype>
 QCborStreamReader::readStringChunk(char *ptr, qsizetype maxlen)
 {
-    auto r = d->readStringChunk(ptr, maxlen);
+    auto r = d->readStringChunk({ptr, maxlen});
     if (r.status == EndOfString && lastError() == QCborError::NoError)
         preparse();
     return r;
 }
 
-QCborStreamReader::StringResult<qsizetype>
-QCborStreamReaderPrivate::readStringChunk(char *ptr, qsizetype maxlen)
+// used by qcborvalue.cpp
+QCborStreamReader::StringResultCode qt_cbor_append_string_chunk(QCborStreamReader &reader, QByteArray *data)
+{
+    return QCborStreamReaderPrivate::appendStringChunk(reader, data);
+}
+
+inline QCborStreamReader::StringResultCode
+QCborStreamReaderPrivate::appendStringChunk(QCborStreamReader &reader, QByteArray *data)
+{
+    auto status = reader.d->readStringChunk(data).status;
+    if (status == QCborStreamReader::EndOfString && reader.lastError() == QCborError::NoError)
+        reader.preparse();
+    return status;
+}
+
+Q_NEVER_INLINE QCborStreamReader::StringResult<qsizetype>
+QCborStreamReaderPrivate::readStringChunk(ReadStringChunk params)
 {
     CborError err;
     size_t len;
@@ -1481,6 +1446,12 @@ QCborStreamReaderPrivate::readStringChunk(char *ptr, qsizetype maxlen)
     if (!ensureStringIteration())
         return result;
 
+    // Note: in the current implementation, the call into TinyCBOR below only
+    // succeeds if we *already* have all the data in memory. That's obvious for
+    // the case of direct memory (no QIODevice), whereas for QIODevices
+    // qt_cbor_decoder_transfer_string() enforces that
+    // QIODevice::bytesAvailable() be bigger than the amount we're about to
+    // read.
 #if 1
     // Using internal TinyCBOR API!
     err = _cbor_value_get_string_chunk(&currentElement, &content, &len, &currentElement);
@@ -1513,19 +1484,84 @@ QCborStreamReaderPrivate::readStringChunk(char *ptr, qsizetype maxlen)
         return result;
     }
 
-    // Read the chunk into the user's buffer.
-    qint64 actuallyRead;
     qptrdiff offset = qptrdiff(content);
-    qsizetype toRead = qsizetype(len);
-    qsizetype left = toRead - maxlen;
-    if (left < 0)
-        left = 0;               // buffer bigger than string
-    else
-        toRead = maxlen;        // buffer smaller than string
-
+    bufferStart += offset;
     if (device) {
         // This first skip can't fail because we've already read this many bytes.
-        device->skip(bufferStart + qptrdiff(content));
+        device->skip(bufferStart);
+    }
+
+    if (params.isString()) {
+        // readString()
+        result.data = readStringChunk_unicode(params, qsizetype(len));
+    } else {
+        // readByteArray() or readStringChunk()
+        result.data = readStringChunk_byte(params, qsizetype(len));
+    }
+
+    if (result.data < 0)
+        return result;      // error
+
+    // adjust the buffers after we're done reading the string
+    bufferStart += len;
+    if (device) {
+        qsizetype remainingInBuffer = buffer.size() - bufferStart;
+
+        if (remainingInBuffer <= 0) {
+            // We've read from the QIODevice more than what was in the buffer.
+            buffer.truncate(0);
+        } else {
+            // There's still data buffered, but we need to move it around.
+            char *ptr = buffer.data();
+            memmove(ptr, ptr + bufferStart, remainingInBuffer);
+            buffer.truncate(remainingInBuffer);
+        }
+
+        bufferStart = 0;
+    }
+
+    preread();
+    result.status = QCborStreamReader::Ok;
+    return result;
+}
+
+inline qsizetype
+QCborStreamReaderPrivate::readStringChunk_byte(ReadStringChunk params, qsizetype len)
+{
+    qint64 actuallyRead;
+    qsizetype toRead = qsizetype(len);
+    qsizetype left = 0;     // bytes from the chunk not copied to the user buffer, to discard
+    char *ptr = nullptr;
+
+    if (params.isPlainPointer()) {
+        left = toRead - params.maxlen_or_type;
+        if (left < 0)
+            left = 0;                           // buffer bigger than string
+        else
+            toRead = params.maxlen_or_type;     // buffer smaller than string
+        ptr = params.ptr;
+    } else if (params.isByteArray()) {
+        // See note above on having ensured there is enough incoming data.
+        auto oldSize = params.array->size();
+        auto newSize = oldSize;
+        if (qAddOverflow<decltype(newSize)>(oldSize, toRead, &newSize)) {
+            handleError(CborErrorDataTooLarge);
+            return -1;
+        }
+        QT_TRY {
+            params.array->resize(newSize);
+        } QT_CATCH (const std::bad_alloc &) {
+            // the distinction between DataTooLarge and OOM is mostly for
+            // compatibility with Qt 5; in Qt 6, we could consider everything
+            // to be OOM.
+            handleError(newSize > MaxByteArraySize ? CborErrorDataTooLarge: CborErrorOutOfMemory);
+            return -1;
+        }
+
+        ptr = const_cast<char *>(params.array->constData()) + oldSize;
+    }
+
+    if (device) {
         actuallyRead = device->read(ptr, toRead);
 
         if (actuallyRead != toRead)  {
@@ -1538,20 +1574,71 @@ QCborStreamReaderPrivate::readStringChunk(char *ptr, qsizetype maxlen)
 
         if (actuallyRead < 0) {
             handleError(CborErrorIO);
-            return result;
+            return -1;
         }
-
-        updateBufferAfterString(offset, len);
     } else {
         actuallyRead = toRead;
-        memcpy(ptr, buffer.constData() + bufferStart + offset, toRead);
-        bufferStart += QByteArray::size_type(offset + len);
+        memcpy(ptr, buffer.constData() + bufferStart, toRead);
     }
 
-    preread();
-    result.data = actuallyRead;
-    result.status = QCborStreamReader::Ok;
-    return result;
+    return actuallyRead;
+}
+
+inline qsizetype
+QCborStreamReaderPrivate::readStringChunk_unicode(ReadStringChunk params, qsizetype utf8len)
+{
+    // See QUtf8::convertToUnicode() a detailed explanation of why this
+    // conversion uses the same number of words or less.
+    QChar *begin = nullptr;
+    if (params.isString()) {
+        QT_TRY {
+            params.string->resize(utf8len);
+        } QT_CATCH (const std::bad_alloc &) {
+            if (utf8len > MaxStringSize)
+                handleError(CborErrorDataTooLarge);
+            else
+                handleError(CborErrorOutOfMemory);
+            return -1;
+        }
+
+        begin = const_cast<QChar *>(params.string->constData());
+    }
+
+    QChar *ptr = begin;
+    QStringConverter::State cs(QStringConverter::Flag::Stateless);
+    if (device == nullptr) {
+        // Easy case: we can decode straight from the buffer we already have
+        ptr = QUtf8::convertToUnicode(ptr, { buffer.constData() + bufferStart, utf8len }, &cs);
+    } else {
+        // read in chunks, to avoid creating large, intermediate buffers
+        constexpr qsizetype StringChunkSize = 16384;
+        qsizetype chunkSize = qMin(StringChunkSize, utf8len);
+        QVarLengthArray<char> chunk(chunkSize);
+
+        cs = { QStringConverter::Flag::ConvertInitialBom };
+        while (utf8len > 0 && cs.invalidChars == 0) {
+            qsizetype toRead = qMin(chunkSize, utf8len);
+            qint64 actuallyRead = device->read(chunk.data(), toRead);
+            if (actuallyRead == toRead)
+                ptr = QUtf8::convertToUnicode(ptr, { chunk.data(), toRead }, &cs);
+
+            if (actuallyRead != toRead) {
+                handleError(CborErrorIO);
+                return -1;
+            }
+            utf8len -= toRead;
+        }
+    }
+
+    if (cs.invalidChars != 0 || cs.remainingChars != 0) {
+        handleError(CborErrorInvalidUtf8TextString);
+        return -1;
+    }
+
+    qsizetype size = ptr - begin;
+    if (params.isString())
+        params.string->truncate(size);
+    return size;
 }
 
 QT_END_NAMESPACE
