@@ -1,43 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#define Q_TEST_QPIXMAPCACHE
 #include "qpixmapcache.h"
 #include "qobject.h"
 #include "qdebug.h"
@@ -180,8 +143,8 @@ bool QPixmapCache::Key::operator ==(const Key &key) const
 
 /*!
     \fn void QPixmapCache::Key::swap(Key &)
-    \internal
     \since 5.6
+    \memberswap{key}
 */
 
 /*!
@@ -289,25 +252,12 @@ bool QPMCache::flushDetachedPixmaps(bool nt)
 {
     auto mc = maxCost();
     const qsizetype currentTotal = totalCost();
+    const qsizetype oldSize = size();
     if (currentTotal)
         setMaxCost(nt ? currentTotal * 3 / 4 : currentTotal - 1);
     setMaxCost(mc);
     ps = totalCost();
-
-    bool any = false;
-    QHash<QString, QPixmapCache::Key>::iterator it = cacheKeys.begin();
-    while (it != cacheKeys.end()) {
-        const auto value = it.value();
-        if (value.isValid() && !contains(value)) {
-            releaseKey(value);
-            it = cacheKeys.erase(it);
-            any = true;
-        } else {
-            ++it;
-        }
-    }
-
-    return any;
+    return size() != oldSize;
 }
 
 void QPMCache::timerEvent(QTimerEvent *)
@@ -326,17 +276,9 @@ void QPMCache::timerEvent(QTimerEvent *)
 
 QPixmap *QPMCache::object(const QString &key) const
 {
-    QPixmapCache::Key cacheKey = cacheKeys.value(key);
-    if (!cacheKey.d || !cacheKey.d->isValid) {
-        const_cast<QPMCache *>(this)->cacheKeys.remove(key);
-        return nullptr;
-    }
-    QPixmap *ptr = QCache<QPixmapCache::Key, QPixmapCacheEntry>::object(cacheKey);
-     //We didn't find the pixmap in the cache, the key is not valid anymore
-    if (!ptr) {
-        const_cast<QPMCache *>(this)->cacheKeys.remove(key);
-    }
-    return ptr;
+    if (const auto it = cacheKeys.find(key); it != cacheKeys.cend())
+        return object(it.value());
+    return nullptr;
 }
 
 QPixmap *QPMCache::object(const QPixmapCache::Key &key) const
@@ -357,6 +299,7 @@ bool QPMCache::insert(const QString& key, const QPixmap &pixmap, int cost)
     // this will create a new key; the old one has been removed
     auto k = insert(pixmap, cost);
     if (k.isValid()) {
+        k.d->stringKey = key;
         cacheKeys[key] = std::move(k);
         return true;
     }
@@ -432,7 +375,11 @@ QPixmapCache::Key QPMCache::createKey()
 void QPMCache::releaseKey(const QPixmapCache::Key &key)
 {
     QPixmapCache::KeyData *keyData = key.d;
-    if (!keyData || keyData->key > keyArraySize || keyData->key <= 0)
+    if (!keyData)
+        return;
+    if (!keyData->stringKey.isNull())
+        cacheKeys.remove(keyData->stringKey);
+    if (keyData->key > keyArraySize || keyData->key <= 0)
         return;
     keyData->key--;
     keyArray[keyData->key] = freeKey;
@@ -459,7 +406,6 @@ void QPMCache::clear()
         killTimer(theid);
         theid = 0;
     }
-    cacheKeys.clear();
 }
 
 QPixmapCache::KeyData* QPMCache::getKeyData(QPixmapCache::Key *key)
@@ -665,14 +611,14 @@ void QPixmapCache::clear()
     }
 }
 
-void QPixmapCache::flushDetachedPixmaps()
+Q_AUTOTEST_EXPORT void qt_qpixmapcache_flush_detached_pixmaps() // for tst_qpixmapcache
 {
     if (!qt_pixmapcache_thread_test())
         return;
     pm_cache()->flushDetachedPixmaps(true);
 }
 
-int QPixmapCache::totalUsed()
+Q_AUTOTEST_EXPORT int qt_qpixmapcache_qpixmapcache_total_used() // for tst_qpixmapcache
 {
     if (!qt_pixmapcache_thread_test())
         return 0;
