@@ -623,17 +623,13 @@ QPolygon QPolygonF::toPolygon() const
 /*!
     \fn void QPolygon::swap(QPolygon &other)
     \since 4.8
-
-    Swaps polygon \a other with this polygon. This operation is very
-    fast and never fails.
+    \memberswap{polygon}
 */
 
 /*!
     \fn void QPolygonF::swap(QPolygonF &other)
     \since 4.8
-
-    Swaps polygon \a other with this polygon. This operation is very
-    fast and never fails.
+    \memberswap{polygon}
 */
 
 /*!
